@@ -5,57 +5,71 @@ import os
 import sys
 import warnings
 import inspect
+import gi
+import time
+
+gi.require_version("Gtk", "3.0")
+from gi.repository import Gtk, Gio, GLib
 
 from PIL import Image
 warnings.simplefilter('ignore', Image.DecompressionBombWarning)
 
 
-def start_job(dir_path, currentfile, progressbar_batch, progressbar_singlefile, outputarea, ocrmypdfsettings):
-	t = threading.Thread(target=batch_ocr, args=(dir_path, progressbar_batch, progressbar_singlefile, outputarea, ocrmypdfsettings, currentfile), daemon=True)
+def start_job(dir_path, progressbar_batch, print_to_textview, ocrmypdfsettings):
+	t = threading.Thread(target=batch_ocr, args=(dir_path, progressbar_batch, print_to_textview, ocrmypdfsettings), daemon=True)
 	t.start()
+	return t
 
-def ocr_run(file_path, ocrmypdfsettings):
+
+def ocr_run(file_path, ocrmypdfsettings, print_to_textview):
 	print(ocrmypdfsettings)
 	#runs ocrmypdf on given file
 	try:
+		print("Start OCR - " + file_path)
+#		ocr = ocrmypdf.ocr(file_path, file_path, **ocrmypdfsettings, plugins=["plugin_progressbar"])
 		ocr = ocrmypdf.ocr(file_path, file_path, **ocrmypdfsettings, plugins=["ocrmypdfgui.plugin_progressbar"])
+		GLib.idle_add(print_to_textview, "OCR complete.\n")
+
 		print("OCR complete.\n")
 		return "OCR complete.\n"
-	except ocrmypdf.exceptions.PriorOcrFoundError:
 
+	except ocrmypdf.exceptions.PriorOcrFoundError:
+		GLib.idle_add(print_to_textview, "Prior OCR - Skipping\n")
 		print("Prior OCR - Skipping\n")
 		return "Prior OCR - Skipping\n"
-	except ocrmypdf.exceptions.EncryptedPdfError:
 
+	except ocrmypdf.exceptions.EncryptedPdfError:
+		GLib.idle_add(print_to_textview, "PDF File is encrypted. Skipping.\n")
 		print("PDF File is encrypted. Skipping.\n")
 		return "PDF File is encrypted. Skipping.\n"
 
 	except ocrmypdf.exceptions.BadArgsError:
+		GLib.idle_add(print_to_textview, "Bad arguments.\n")
 		print("Bad arguments.\n")
 
 	except:
-		e = sys.exc_info()[0]
+		e = sys.exc_info()
+		GLib.idle_add(print_to_textview, str(e))
+		GLib.idle_add(print_to_textview, "\n")
+		#time.sleep(0.2)
 		print(e)
 		return "Error.\n"
 
-def batch_ocr(dir_path, progressbar_batch, progressbar_singlefile, outputarea, ocrmypdfsettings, currentfile):
+def batch_ocr(dir_path, progressbar_batch, print_to_textview, ocrmypdfsettings):
 	# walks through given path and uses OCR Function on every pdf in path
-	progressbar_batch.set(0.0)	#resets Progressbar
+	GLib.idle_add(progressbar_batch, 0.0) #resets progressbar_batch
 	progress_precision = 0.0
-
+	percent_step = 0.0
 	if(os.path.isfile(dir_path)==True):
 		#Run OCR on single file
 		file_ext = os.path.splitext(dir_path)[1]
 		if file_ext == '.pdf':
 
 			print("Path:" + dir_path + "\n")
-			outputarea.insert("end", "File: " + dir_path + " - ")
-			outputarea.see("end")
-			currentfile.set("Current File:" + dir_path )
-			result = ocr_run(dir_path, ocrmypdfsettings)
-			outputarea.insert("end", result)
-			outputarea.see("end")
-			progressbar_batch.set(100)
+			GLib.idle_add(print_to_textview, "File: " + dir_path + " - ")
+
+			result = ocr_run(dir_path, ocrmypdfsettings, print_to_textview)
+			GLib.idle_add(progressbar_batch, 1.0) #Sets progressbar_batch to 100%
 	elif(os.path.isdir(dir_path)==True):
 		number_of_files = 0
 		for dir_name, subdirs, file_list in os.walk(dir_path):
@@ -65,7 +79,8 @@ def batch_ocr(dir_path, progressbar_batch, progressbar_singlefile, outputarea, o
 					number_of_files=number_of_files+1
 
 			if number_of_files >0:
-				percent = 100/number_of_files
+				percent_step = 1/number_of_files	#1 = 100%
+				print("percent_step: " + str(percent_step) + " - " + "number_of_files: " + str(number_of_files))
 		for dir_name, subdirs, file_list in os.walk(dir_path):
 			print(file_list)
 
@@ -75,18 +90,13 @@ def batch_ocr(dir_path, progressbar_batch, progressbar_singlefile, outputarea, o
 					full_path = dir_name + '/' + filename
 
 					print("Path:" + full_path + "\n")
-					currentfile.set("Current File:" + full_path )
-					outputarea.insert("end", "File: " + full_path + " - ")
-					outputarea.see("end")
-
-					result = ocr_run(full_path, ocrmypdfsettings)
-					outputarea.insert("end", result)
-					outputarea.see("end")
-
-					progress_precision = progress_precision + percent
+					GLib.idle_add(print_to_textview, "File: " + full_path + " - ")
+					result = ocr_run(full_path, ocrmypdfsettings, print_to_textview)
+					progress_precision = progress_precision + percent_step #necessary to hit 100 by incrementing
 					print(progress_precision)
-					progressbar_batch.set(round(progress_precision))
-					progressbar_singlefile.set(0.0)
+					GLib.idle_add(progressbar_batch, progress_precision) #sets progressbar_batch to current progress
+
+
 
 	else:
 		print("Error")
